# -*- ruby -*-

begin
  require './dl'
rescue LoadError
  require 'dl'
end

def assert(label, ty, *conds)
  cond = !conds.include?(false)
  if( cond )
    printf("succeed in `#{label}'\n")
  else
    case ty
    when :may
      printf("fail in `#{label}' ... expected\n")
    when :must
      printf("fail in `#{label}' ... unexpected\n")
    when :raise
      raise(RuntimeError, "fail in `#{label}'")
    end
  end
end

def debug(*xs)
  if( $DEBUG )
    xs.each{|x|
      p x
    }
  end
end

print("VERSION       = #{DL::VERSION}\n")
print("MAJOR_VERSION = #{DL::MAJOR_VERSION}\n")
print("MINOR_VERSION = #{DL::MINOR_VERSION}\n")
print("\n")
print("MAX_ARG   = #{DL::MAX_ARG}\n")
print("MAX_CBARG = #{DL::MAX_CBARG}\n")
print("MAX_CBENT = #{DL::MAX_CBENT}\n")
print("\n")
print("DL::FREE = #{DL::FREE.inspect}\n")
print("\n")

$LIB = nil
if( !$LIB && File.exist?("libtest.so") )
  $LIB = "./libtest.so"
end
if( !$LIB && File.exist?("test/libtest.so") )
  $LIB = "./test/libtest.so"
end

DL.dlopen($LIB){|h|
  c2i = h["test_c2i","IC"]
  debug c2i
  r,rs = c2i[?a]
  debug r,rs
  assert("c2i", :may, r == ?a)

  i2c = h["test_i2c","CI"]
  debug i2c
  r,rs = i2c[?a]
  debug r,rs
  assert("i2c", :may, r == ?a)

  lcc = h["test_lcc","LCC"]
  debug lcc
  r,rs = lcc[1,2]
  assert("lcc", :may, r == 3)

  f2d = h["test_f2d","DF"]
  debug f2d
  r,rs = f2d[20.001]
  debug r,rs
  assert("f2d", :may, r.to_i == 20)

  d2f = h["test_d2f","FD"]
  debug d2f
  r,rs = d2f[20.001]
  debug r,rs
  assert("d2f", :may, r.to_i == 20)

  strlen = h["test_strlen","IS"]
  debug strlen
  r,rs = strlen["0123456789"]
  debug r,rs
  assert("strlen", :must, r == 10)

  isucc = h["test_isucc","II"]
  debug isucc
  r,rs = isucc[2]
  debug r,rs
  assert("isucc", :must, r == 3)

  lsucc = h["test_lsucc","LL"]
  debug lsucc
  r,rs = lsucc[10000000]
  debug r,rs
  assert("lsucc", :must, r == 10000001)

  succ = h["test_succ","0l"]
  debug succ
  r,rs = succ[0]
  debug r,rs
  assert("succ", :must, rs[0] == 1)

  arylen = h["test_arylen","IA"]
  debug arylen
  r,rs = arylen[["a","b","c","d",nil]]
  debug r,rs
  assert("arylen", :must, r == 4)

  arylen = h["test_arylen","IP"]
  debug arylen
  r,rs = arylen[["a","b","c","d",nil]]
  debug r,rs
  assert("arylen", :must, r == 4)

  append = h["test_append","0aIS"]
  debug append
  r,rs = append[["a","b","c"],3,"x"]
  debug r,rs
  assert("append", :must, rs[0].to_a('S',3) == ["ax","bx","cx"])

  strcat = h["test_strcat","SsS"]
  debug strcat
  r,rs = strcat["abc","x"]
  debug r,rs
  assert("strcat", :must, rs[0].to_s == "abcx")

  init = h["test_init","0iP"]
  debug init
  argc = 3
  argv = ["arg0","arg1","arg2"].to_ptr
  r,rs = init[argc, argv.ref]
  debug r,rs
}


h = DL.dlopen($LIB)

sym_open = h["test_open", "PSS"]
sym_gets = h["test_gets", "SsIP"]
sym_close = h["test_close", "0P"]
debug sym_open,sym_gets,sym_close

fp,rs = sym_open["extconf.rb", "r"]
File.open("extconf.rb"){|f|
  $line = f.gets
}
if( fp )
  fp.free = sym_close
  r,rs = sym_gets[" " * 256, 256, fp]
  debug r,rs
  assert("open,gets", :must, rs[0] == $line)
else
  assert("open,gets", :must, $line == nil)
end


callback1 = h["test_callback1"]
debug callback1
r,rs = h["test_call_func1", "IP"][callback1]
debug r,rs
assert("callback1", :must, r == 1)


callback2 = DL.set_callback("LLP", 0){|arg1,arg2|
  ptr = arg2 # DL::PtrData.new(arg2)
  msg = ptr.to_s
  print("user defined callback function",
	"(err = #{arg1}, msg = '#{msg}')\n")
  2
}
debug callback2
r,rs = h["test_call_func1", "IP"][callback2]
debug r,rs
assert("callback2", :must, r == 2)


ptr = DL.malloc(DL.sizeof('CL'))
ptr.struct!("CL", :c, :l)
ptr[:c] = 0
ptr[:l] = 0
r,rs = h["test_fill_test_struct","0PIL"][ptr,100,1000]
debug r,rs
assert("fill_test_struct", :must, ptr[:c] == 100, ptr[:l] == 1000)


r,rs = h["test_alloc_test_struct", "PIL"][100,200]
r.free = DL::FREE
r.struct!("CL", :c, :l)
assert("alloc_test_struct", :must, r[:c] == 100, r[:l] == 200)
